local status_ok, which_key = pcall(require, "which-key")
if not status_ok then
  return
end

local dap = require('dap')
local dap_ui = require('dapui')
local jdtls = require('jdtls')
local zen = require('zen-mode')
local buffer = vim.lsp.buf
local tele = require('telescope.builtin')
local gs = require("gitsigns")
local mason_ui = require('mason.ui')
local harpoon_mark = require('harpoon.mark')
local harpoon_ui = require('harpoon.ui')
local baleia = require('baleia').setup {}
local theme = require("nise.theme")
local neotest = require("neotest")

which_key.setup {
  window = {
    border = "rounded",
  },
  key_labels = {
    ["<space>"] = "SPC"
  },
  icons = {
    group = "󰙅 "
  }
}

local opts = {
  mode = "n",
  prefix = "<leader>",
  buffer = nil,
  silent = true,
  noremap = true,
  nowait = true
}


local file_picker_layout = { preview_height = 0, prompt_position = 'top' }


which_key.register({
  c = { theme.toggle, "Toggle between light and dark themes" },
  z = { zen.toggle, "Toggle zen mode" },
  u = { vim.cmd.UndotreeToggle, "Toggle undo tree" },
  g = {
    name = "Git",
    d = { gs.preview_hunk, "Show inline diff" },
    u = { gs.reset_hunk, "Undo hunk" },
    D = { gs.diffthis, "Show file diff" },
    j = { gs.next_hunk, "Go to next changed hunk" },
    k = { gs.prev_hunk, "Go to previous changed hunk" },
    t = { gs.next_hunk, "Go to next changed hunk" },
    s = { gs.prev_hunk, "Go to previous changed hunk" },
    b = { gs.toggle_current_line_blame, "Toggle blame for current line" },
    B = { function() gs.blame_line { full = true } end, "Blame current line" },
    h = { tele.git_commits, "Show workdir history" }
  },
  l = {
    name = "LSP",
    a = { buffer.code_action, "Code Action" },
    n = { buffer.rename, "Rename symbol" },
    h = { buffer.hover, "Show documentation" },
    S = { buffer.signature_help, "Show signature help" },
    d = { buffer.definition, "Go to definition" },
    i = { buffer.implementation, "Go to implementation" },
    f = { function() buffer.format { async = true } end, "Reformat current buffer" },
    r = { function() tele.lsp_references({ layout_strategy = 'vertical' }) end, "Find references" },
    s = { tele.lsp_document_symbols, "List document symbols" },
    v = { tele.diagnostics, "Show diagnostics" },
    w = { buffer.workspace_symbol, "List workspace symbols" },
    l = { mason_ui.open, "Open LSP installer window" }
  },
  f = {
    name = "Find",

    f = { function()
      tele.git_files({
        layout_strategy = 'vertical',
        layout_config = file_picker_layout,
        sorting_strategy = 'ascending'
      })
    end, "Find in git files" },

    F = { function()
      tele.find_files({
        hidden = true,
        no_ignore = true,
        layout_strategy = 'vertical',
        layout_config = file_picker_layout,
        sorting_strategy = 'ascending'
      })
    end, "Find all files" },

    o = { tele.oldfiles, "Open old files" },

    g = { function() tele.live_grep({ layout_strategy = 'vertical', }) end,
      "Find text" },
    G = { function()
      tele.live_grep({
        layout_strategy = 'vertical',
        additional_args = { '--no-ignore', '--hidden' }
      })
    end, "Find text everywhere" },

    c = { tele.grep_string, "Find text under cursor" },
    b = { function() tele.buffers({ path_display = { 'truncate' } }) end, "Find buffer" },
    h = { tele.help_tags, "Find help tag" },
    ['/'] = { function()
      tele.current_buffer_fuzzy_find({ sorting_strategy = 'ascending', layout_config = { prompt_position = 'top' } })
    end, "Find in current buffer" }
  },
  t = {
    name = "Tree",
    t = { vim.cmd.NvimTreeToggle, "Toggle file tree" },
    f = { vim.cmd.NvimTreeFindFileToggle, "Find file in tree" },
    n = { vim.cmd.NnnExplorer, "Open nnn" }
  },
  d = {
    name = 'Debug',
    m = { function()
      if vim.bo.filetype == 'java' then
        jdtls.test_nearest_method()
        dap.repl.open({ width = 60 }, 'vsplit')
      else
        neotest.run.run({ strategy = "dap" })
        neotest.summary.open()
      end
    end, "Test nearest method" },
    c = { function()
      if vim.bo.filetype == 'java' then
        jdtls.test_class()
        dap.repl.open({ width = 60 }, 'vsplit')
      else
        neotest.run.run(vim.fn.expand("%"))
        neotest.summary.open()
      end
    end, "Test class" },
    b = { dap.toggle_breakpoint, "Toggle breakpoint" },
    r = { dap.repl.open, "Open repl" },
    f = { dap.restart_frame, "Restart frame" },
    o = { function()
      dap_ui.toggle({ layout = 1 })
    end, "Toggle dap ui" },
    t = { function()
      dap_ui.toggle({ layout = 2 })
    end, "Show terminal" }
  },
  C = { function() baleia.once(vim.fn.bufnr('%')) end, "Parse ANSI escape sequences" },
  h = {
    name = 'harpoon',
    a = { harpoon_mark.add_file, 'Add file to harpoon' },
    m = { harpoon_ui.toggle_quick_menu, 'Show harpoon menu' },
    n = { harpoon_ui.nav_next, 'Navigate to next harpoon' },
    p = { harpoon_ui.nav_prev, 'Navigate to previous harpoon' },
    u = { function() harpoon_ui.nav_file(1) end, 'Navigate to harpoon #1' },
    s = { function() harpoon_ui.nav_file(1) end, 'Navigate to harpoon #1' },
    i = { function() harpoon_ui.nav_file(2) end, 'Navigate to harpoon #2' },
    d = { function() harpoon_ui.nav_file(2) end, 'Navigate to harpoon #2' },
    e = { function() harpoon_ui.nav_file(3) end, 'Navigate to harpoon #3' },
    f = { function() harpoon_ui.nav_file(3) end, 'Navigate to harpoon #3' },
    [','] = { function() harpoon_ui.nav_file(4) end, 'Navigate to harpoon #4' },
    g = { function() harpoon_ui.nav_file(4) end, 'Navigate to harpoon #4' },
  }
}, opts)

which_key.register({
    l = {
      name = "LSP",
      a = { buffer.code_action, "Code Action" },
      f = { vim.lsp.buf.format, "Reformat current selection" },
    },
    d = {
      name = "Debug",
      e = { dap_ui.eval, "Evaluate selected expression" }
    },
  },
  {
    mode = "v",
    prefix = "<leader>"
  }
)

which_key.register({
    ['<F7>'] = { dap.step_into, "Step into" },
    ['<F8>'] = { dap.step_over, "Step over" },
    ["<F9>"] = { dap.continue, "Continue" },
    ["<F10>"] = { dap.terminate, "Terminate" },
    ['<F11>'] = { dap.step_out, "Step out" }
  },
  {
    mode = "n",
    prefix = ""
  }
)
